
#include "stm32f746xx.h"
#include "stm32f7xx_hal.h"
#include "stm32746g_discovery.h"
#include "stm32746g_discovery_lcd.h"
#include "stm32746g_discovery_audio.h"
#include "stm32746g_discovery_ts.h"
#include "arm_const_structs.h"
#include "arm_math.h"
#include "asa.h"
#include "asa_filt_coefs.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#define AUDIO_IN_BUFFER_LENGTH 4096  //length of buffer in half words for received audio data
#define THIRD_OCTAVE_FILTERS_NUM 30  //number of third-octave BP filters
#define DISPLAY_REFRESH_PERIOD 100  //period in ms for display refreshing
#define FREQ_AXIS_REFRESH_PERIOD 4000
#define TS_SAMPLING_PERIOD 200  //touch screen sampling period
#define MICPHONE_CALIB_CONST ((float)(55.0))  //calibration constant for digital MEMS microphone
#define SPL_EDGE_OF_HEARING ((float)(0.00002)) //edge of human hearing in SPL[Pa]
#define LCD_WIDTH  (RK043FN48H_WIDTH)   //LCD pixel width
#define LCD_HEIGHT (RK043FN48H_HEIGHT)  //LCD pixel height
#define SAMPLING_FREQUENCY ((uint16_t)(44100))

#define SPL_UPPER_LIMIT_DEFAULT ((float)(120.0)) //default upper limit of SPL[dB] on display
#define SPL_LOWER_LIMIT_DEFAULT ((float)(0.0))  //default lower limit of SPL[dB] on display
#define VOLTAGE_UPPER_LIMIT_DEFAULT ((float)(80.0)) //default upper limit of V[dBmV] on display
#define VOLTAGE_LOWER_LIMIT_DEFAULT ((float)(-40.0))  //default lower limit of V[dBmV] on display
#define FREQUENCY_UPPER_LIMIT_DEFAULT ((float)(SAMPLING_FREQUENCY/2)) //default upper limit of V[dBmV] on display
#define FREQUENCY_LOWER_LIMIT_DEFAULT ((float)(0.0))  //default lower limit of V[dBmV] on display
#define LCD_THIRD_OCTAVE_SPECTRAL_WIDTH ((uint16_t)11)  //pixel width of one spectral column on LCD
#define LCD_THIRD_OCTAVE_GAP_WIDTH ((uint16_t)2)  //pixel width of gap between spectral columns on LCD

typedef enum
{
  ASA_INT2FLOAT_MODE_NORMAL,
	ASA_INT2FLOAT_MODE_FFT,
	ASA_INT2FLOAT_MODE_DEC2
} ASA_int2float_mode_typedef;

typedef enum
{
	ASA_FREQ_IN_172Hz,
	ASA_FREQ_IN_345Hz,
	ASA_FREQ_IN_689Hz,
	ASA_FREQ_IN_1378Hz,
	ASA_FREQ_IN_2756Hz,
	ASA_FREQ_IN_5513Hz,
	ASA_FREQ_IN_11025Hz,
	ASA_FREQ_IN_22050Hz,
	ASA_FREQ_IN_44100Hz
} ASA_freq_in_typedef;

typedef enum
{
	ASA_DECIMATE_BAND_345Hz = 4,
	ASA_DECIMATE_BAND_689Hz = 7,
	ASA_DECIMATE_BAND_1378Hz = 10,
	ASA_DECIMATE_BAND_2756Hz = 13,
	ASA_DECIMATE_BAND_5513Hz = 16,
	ASA_DECIMATE_BAND_11025Hz = 19,
	ASA_DECIMATE_BAND_22050Hz = 22,
	ASA_DECIMATE_BAND_44100Hz = 25
} ASA_decimate_band_typedef;

typedef enum
{
	ASA_AUDIO_INPUT_MICROPHONE = 0,
	ASA_AUDIO_INPUT_AUDIOJACK = 1
} ASA_audio_input_typedef;

static struct
{
	uint32_t event : 1;

	uint32_t display_refresh : 1;
	uint32_t freq_axis_refresh : 1;
	uint32_t ts_read_state : 1;
	uint32_t transfer_complete : 1;
	uint32_t half_transfer : 1;
	uint32_t ts_touch : 1;
} ASA_flags;

static struct
{
	uint32_t audio_input : 1;
} ASA_general_settings;

static struct
{
	int16_t y_max;
	int16_t y_min;
	uint32_t x_max;
	uint32_t x_min;
} ASA_axis_settings;

static struct
{
	uint16_t upper;
	uint16_t lower;
	uint16_t left;
	uint16_t right;
} ASA_graph_limits;

//audio buffers
static int16_t i16_audio_in_data[AUDIO_IN_BUFFER_LENGTH] = {0};
static float f_audio_in_data[AUDIO_IN_BUFFER_LENGTH/2] = {0.0};
static float third_octave_spectrum[THIRD_OCTAVE_FILTERS_NUM] = {0};

/*Draw x,y axises on LCD*/
static void ASA_create_xy_axis(void)
{
	char* third_octave_nom_freq[THIRD_OCTAVE_FILTERS_NUM] = {"25", "31.5", "40", "50", "63", "80",
																																 "100", "125", "160", "200", "250", "315",
																																 "400", "500", "630", "800", "1k", "1.25k",
																																 "1.6k", "2k", "2.5k", "3.15k", "4k", "5k",
																																 "6.3k", "8k", "10k", "12.5k", "16k", "20k", };
	uint8_t y_unit[10];
	uint8_t y_quantity[10];
	uint8_t x_quantity_unit[10];
	uint8_t num[5];

	if(ASA_general_settings.audio_input == ASA_AUDIO_INPUT_MICROPHONE)
	{
		strcpy(y_quantity, " SPL");
		strcpy(y_unit, " [dB]");
	}
	else
	{
		strcpy(y_quantity, "  V");
		strcpy(y_unit, "[dBmV]");
	}
	strcpy(x_quantity_unit, "f[Hz]");

	BSP_LCD_SetBackColor(LCD_COLOR_BLACK);
	BSP_LCD_SetTextColor(LCD_COLOR_WHITE);
	BSP_LCD_SetFont(&Font12);

	//draw quantities and units
	BSP_LCD_DisplayStringAt((uint16_t)445, (uint16_t)258, x_quantity_unit, LEFT_MODE);
	BSP_LCD_DisplayStringAt((uint16_t)4, (uint16_t)1, y_quantity, LEFT_MODE);
	BSP_LCD_DisplayStringAt((uint16_t)1, (uint16_t)13, y_unit, LEFT_MODE);

	//draw xy axises
	BSP_LCD_DrawHLine((uint16_t)47, (uint16_t)256, (uint16_t)394);
	BSP_LCD_DrawVLine((uint16_t)47, (uint16_t)26, (uint16_t)231);

	//draw numbers at xy axises
	BSP_LCD_SetFont(&Font8);
	for(uint8_t i = 0; i <= (ASA_axis_settings.y_max - ASA_axis_settings.y_min)/10; i++)
	{
		sprintf(num, "%d", ASA_axis_settings.y_min + i*10);
		BSP_LCD_DisplayStringAt((uint16_t)(30), (uint16_t)256-8-i*17, num, LEFT_MODE);
	}

	for(uint8_t i = 0; i < THIRD_OCTAVE_FILTERS_NUM/2; i++)
	{
		BSP_LCD_DisplayStringAt((uint16_t)(49 + i*2*13), (uint16_t)260, third_octave_nom_freq[2*i], LEFT_MODE);
	}

	ASA_graph_limits.left = 50;
	ASA_graph_limits.right = 390+50-1;
	ASA_graph_limits.upper = 256-13*17+8;
	ASA_graph_limits.lower = 256-1;



	return;
}

/*Convert samples int16_t to float*/
static void ASA_int2float_conv(float *buff_out, int16_t *buff_in, uint32_t size, ASA_int2float_mode_typedef mode)
{
	switch(mode)
	{
		case ASA_INT2FLOAT_MODE_NORMAL:
			for(uint32_t i = 0; i < size; i++)
				buff_out[i] = (float)buff_in[i];
			break;
		case ASA_INT2FLOAT_MODE_FFT:
			for(uint32_t i = 0; i < size; i++)
				if(i & (uint32_t)1)
					buff_out[i] = (float)0.0;
				else
					buff_out[i] = (float)buff_in[i];
			break;
		case ASA_INT2FLOAT_MODE_DEC2:
			size >>= 1;
			for(uint32_t i = 0; i < size; i++)
				buff_out[i] = (float)buff_in[2*i];
			break;
		default:
			for(uint32_t i = 0; i < size; i += 2)
				buff_out[i] = (float)buff_in[i];
			break;
	}

	return;
}

/*Filter input signal by given third-octave filter*/
static void ASA_third_octave_filtering(float *src, float *dest, uint8_t band_num, uint32_t size)
{
	static float third_octave_filt_state[30][12] = {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
																									{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};

	const arm_biquad_casd_df1_inst_f32 S[30] = {{(uint32_t)3, third_octave_filt_state[0], third_octave_filt_coefs[0]},
	                                            {(uint32_t)3, third_octave_filt_state[1], third_octave_filt_coefs[1]},
	                                            {(uint32_t)3, third_octave_filt_state[2], third_octave_filt_coefs[2]},
	                                            {(uint32_t)3, third_octave_filt_state[3], third_octave_filt_coefs[3]},
	                                            {(uint32_t)3, third_octave_filt_state[4], third_octave_filt_coefs[4]},
	                                            {(uint32_t)3, third_octave_filt_state[5], third_octave_filt_coefs[5]},
	                                            {(uint32_t)3, third_octave_filt_state[6], third_octave_filt_coefs[6]},
	                                            {(uint32_t)3, third_octave_filt_state[7], third_octave_filt_coefs[7]},
	                                            {(uint32_t)3, third_octave_filt_state[8], third_octave_filt_coefs[8]},
	                                            {(uint32_t)3, third_octave_filt_state[9], third_octave_filt_coefs[9]},
	                                            {(uint32_t)3, third_octave_filt_state[10], third_octave_filt_coefs[10]},
	                                            {(uint32_t)3, third_octave_filt_state[11], third_octave_filt_coefs[11]},
	                                            {(uint32_t)3, third_octave_filt_state[12], third_octave_filt_coefs[12]},
	                                            {(uint32_t)3, third_octave_filt_state[13], third_octave_filt_coefs[13]},
	                                            {(uint32_t)3, third_octave_filt_state[14], third_octave_filt_coefs[14]},
	                                            {(uint32_t)3, third_octave_filt_state[15], third_octave_filt_coefs[15]},
	                                            {(uint32_t)3, third_octave_filt_state[16], third_octave_filt_coefs[16]},
	                                            {(uint32_t)3, third_octave_filt_state[17], third_octave_filt_coefs[17]},
	                                            {(uint32_t)3, third_octave_filt_state[18], third_octave_filt_coefs[18]},
	                                            {(uint32_t)3, third_octave_filt_state[19], third_octave_filt_coefs[19]},
	                                            {(uint32_t)3, third_octave_filt_state[20], third_octave_filt_coefs[20]},
	                                            {(uint32_t)3, third_octave_filt_state[21], third_octave_filt_coefs[21]},
	                                            {(uint32_t)3, third_octave_filt_state[22], third_octave_filt_coefs[22]},
	                                            {(uint32_t)3, third_octave_filt_state[23], third_octave_filt_coefs[23]},
	                                            {(uint32_t)3, third_octave_filt_state[24], third_octave_filt_coefs[24]},
	                                            {(uint32_t)3, third_octave_filt_state[25], third_octave_filt_coefs[25]},
	                                            {(uint32_t)3, third_octave_filt_state[26], third_octave_filt_coefs[26]},
	                                            {(uint32_t)3, third_octave_filt_state[27], third_octave_filt_coefs[27]},
	                                            {(uint32_t)3, third_octave_filt_state[28], third_octave_filt_coefs[28]},
	                                            {(uint32_t)3, third_octave_filt_state[29], third_octave_filt_coefs[29]}};

	arm_biquad_cascade_df1_f32(&S[band_num], src, dest, size);

	for(uint32_t i = 0; i < size; i++)
		dest[i] = dest[i] * third_octave_filt_gains[band_num];

	return;
}

/*Performs mean square of input signal for given third-octave band and sampling frequency*/
static void ASA_make_mean_square(float *src, float *dest, uint8_t band_num, ASA_freq_in_typedef freq_in, uint32_t size)
{
	static float avrg_filt_state[30][2] = {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0},
																				 {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0},
																				 {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0},
																				 {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0},
																				 {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0},
																				 {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}};
	float xn = 0.0;

	for(uint32_t i = 0; i < size; i++)
	{
		xn = src[i] * src[i];
		dest[i] = avrg_filt_coefs[freq_in][0] * xn +
				      avrg_filt_coefs[freq_in][1] * avrg_filt_state[band_num][0] +
							avrg_filt_coefs[freq_in][2] * avrg_filt_state[band_num][1];
		avrg_filt_state[band_num][1] = dest[i];
		avrg_filt_state[band_num][0] = xn;
	}

	return;
}

/*Performs decimation by 2 for input signal of given sampling frequency*/
static void ASA_decimate_2x(float *src, float *dest, ASA_freq_in_typedef freq_in, uint32_t size)
{
	static float decimation_filt_state[8][12];
	const arm_biquad_casd_df1_inst_f32 S[8] = {{(uint32_t)3, decimation_filt_state[0], decimation_filt_coefs},
																						 {(uint32_t)3, decimation_filt_state[1], decimation_filt_coefs},
																						 {(uint32_t)3, decimation_filt_state[2], decimation_filt_coefs},
																						 {(uint32_t)3, decimation_filt_state[3], decimation_filt_coefs},
																						 {(uint32_t)3, decimation_filt_state[4], decimation_filt_coefs},
																						 {(uint32_t)3, decimation_filt_state[5], decimation_filt_coefs},
																						 {(uint32_t)3, decimation_filt_state[6], decimation_filt_coefs},
																						 {(uint32_t)3, decimation_filt_state[7], decimation_filt_coefs}};

	arm_biquad_cascade_df1_f32(&S[freq_in], src, dest, size);

	size >>= 1;

	for(uint32_t i = 0; i < size; i++)
		dest[i] = dest[2*i] * decimation_filt_gain;

	return;
}

/*Create third octave spectrum for samples received by DMA*/
static void ASA_make_third_octave_spectrum(void)
{
	float buff_temp[AUDIO_IN_BUFFER_LENGTH/4];
	uint32_t size = AUDIO_IN_BUFFER_LENGTH/4;
	ASA_freq_in_typedef freq_in = ASA_FREQ_IN_44100Hz;

	for(int8_t band_num = THIRD_OCTAVE_FILTERS_NUM - 1; band_num >= 0; band_num--)
	{
	  ASA_third_octave_filtering(f_audio_in_data, buff_temp, band_num, size);
	  ASA_make_mean_square(buff_temp, buff_temp, band_num, freq_in, size);

	  if(ASA_general_settings.audio_input == ASA_AUDIO_INPUT_MICROPHONE)
	    third_octave_spectrum[band_num] = 10*log10(buff_temp[size - 1]/SPL_EDGE_OF_HEARING/MICPHONE_CALIB_CONST);
	  else
	    third_octave_spectrum[band_num] = 10*log10(buff_temp[size - 1]/32768.0*1650.0);

	  //decision for decimations
	  switch(band_num)
	  {
	  case ASA_DECIMATE_BAND_345Hz:
	  	freq_in = ASA_FREQ_IN_172Hz;
	  	ASA_decimate_2x(f_audio_in_data, f_audio_in_data, freq_in, size);
	  	size >>= 1;
	  	break;
	  case ASA_DECIMATE_BAND_689Hz:
	  	freq_in = ASA_FREQ_IN_345Hz;
	  	ASA_decimate_2x(f_audio_in_data, f_audio_in_data, freq_in, size);
	  	size >>= 1;
	  	break;
	  case ASA_DECIMATE_BAND_1378Hz:
	  	freq_in = ASA_FREQ_IN_689Hz;
	  	ASA_decimate_2x(f_audio_in_data, f_audio_in_data, freq_in, size);
	  	size >>= 1;
	  	break;
	  case ASA_DECIMATE_BAND_2756Hz:
	  	freq_in = ASA_FREQ_IN_1378Hz;
	  	ASA_decimate_2x(f_audio_in_data, f_audio_in_data, freq_in, size);
	  	size >>= 1;
	  	break;
	  case ASA_DECIMATE_BAND_5513Hz:
	  	freq_in = ASA_FREQ_IN_2756Hz;
	  	ASA_decimate_2x(f_audio_in_data, f_audio_in_data, freq_in, size);
	  	size >>= 1;
	  	break;
	  case ASA_DECIMATE_BAND_11025Hz:
	  	freq_in = ASA_FREQ_IN_5513Hz;
	  	ASA_decimate_2x(f_audio_in_data, f_audio_in_data, freq_in, size);
	  	size >>= 1;
	  	break;
	  case ASA_DECIMATE_BAND_22050Hz:
	  	freq_in = ASA_FREQ_IN_11025Hz;
	  	ASA_decimate_2x(f_audio_in_data, f_audio_in_data, freq_in, size);
	  	size >>= 1;
	  	break;
	  case ASA_DECIMATE_BAND_44100Hz:
	  	freq_in = ASA_FREQ_IN_22050Hz;
	  	ASA_decimate_2x(f_audio_in_data, f_audio_in_data, freq_in, size);
	  	size >>= 1;
	  	break;
	  default:
	  	break;
	  }
	}

	return;
}

/*Refresh spectrum on LCD*/
static void ASA_display_refresh(void)
{
	const float vertical_scale_factor = ((float)(ASA_graph_limits.lower - ASA_graph_limits.upper + 1))/((float)(ASA_axis_settings.y_max - ASA_axis_settings.y_min));

	for(uint8_t i = 0; i < 30; i++)
	{

		if(third_octave_spectrum[i] < (float)ASA_axis_settings.y_min)
			third_octave_spectrum[i] = (float)ASA_axis_settings.y_min;
		else if(third_octave_spectrum[i] > (float)ASA_axis_settings.y_max)
			third_octave_spectrum[i] = (float)ASA_axis_settings.y_max;

		third_octave_spectrum[i] -= (float)ASA_axis_settings.y_min;

		BSP_LCD_SetTextColor(LCD_COLOR_BLACK);
		BSP_LCD_FillRect(((uint16_t)i)*(LCD_THIRD_OCTAVE_GAP_WIDTH + LCD_THIRD_OCTAVE_SPECTRAL_WIDTH) + ASA_graph_limits.left,
				             (uint16_t)ASA_graph_limits.upper,
										 LCD_THIRD_OCTAVE_SPECTRAL_WIDTH,
										 ASA_graph_limits.lower - ASA_graph_limits.upper + 1 - (uint16_t)(vertical_scale_factor*third_octave_spectrum[i]));
		BSP_LCD_SetTextColor(LCD_COLOR_YELLOW);
		BSP_LCD_FillRect(((uint16_t)i)*(LCD_THIRD_OCTAVE_GAP_WIDTH + LCD_THIRD_OCTAVE_SPECTRAL_WIDTH) + ASA_graph_limits.left,
				             ASA_graph_limits.lower + 1 - (uint16_t)(vertical_scale_factor*third_octave_spectrum[i]),
										 LCD_THIRD_OCTAVE_SPECTRAL_WIDTH,
										 (uint16_t)(vertical_scale_factor*third_octave_spectrum[i]));
	}

	return;
}

/*Refresh freq axis on LCD*/
static void ASA_freq_axis_refresh(void)
{
	const char* third_octave_nom_freq[THIRD_OCTAVE_FILTERS_NUM] = {"25", "31.5", "40", "50", "63", "80",
																																 "100", "125", "160", "200", "250", "315",
																																 "400", "500", "630", "800", "1k", "1.25k",
																															 	 "1.6k", "2k", "2.5k", "3.15k", "4k", "5k",
																																 "6.3k", "8k", "10k", "12.5k", "16k", "20k", };
	static uint8_t state = 0;
	BSP_LCD_SetTextColor(LCD_COLOR_BLACK);
	BSP_LCD_FillRect(49, 260, 392, 10);
	BSP_LCD_SetTextColor(LCD_COLOR_WHITE);

	if(state)
	{
		state = 0;
		for(uint8_t i = 0; i < THIRD_OCTAVE_FILTERS_NUM/2; i++)
		{
			BSP_LCD_DisplayStringAt((uint16_t)(49 + i*2*13), (uint16_t)260, third_octave_nom_freq[2*i], LEFT_MODE);
		}

	}
	else
	{
		state = 1;
		for(uint8_t i = 0; i < THIRD_OCTAVE_FILTERS_NUM/2; i++)
		{
			BSP_LCD_DisplayStringAt((uint16_t)(49 + i*2*13 + 13), (uint16_t)260, third_octave_nom_freq[2*i + 1], LEFT_MODE);
		}
	}

	return;
}

/*Changes audio input*/
static void ASA_change_input(void)
{
	static uint8_t state = 0;
	BSP_LCD_Clear(LCD_COLOR_BLACK);
	if(state)
	{
		state = 0;
		BSP_AUDIO_IN_Stop(CODEC_PDWN_HW);
		BSP_AUDIO_IN_DeInit();
		/*Initialize audio for recording from digital microphones: stereo, gain 80% (0dB), sampling frequency 44.1 kHz*/
		BSP_AUDIO_IN_Init(INPUT_DEVICE_INPUT_LINE_1, (uint8_t) 80, AUDIO_FREQUENCY_44K);
		/*Start recording*/
		BSP_AUDIO_IN_Record((uint16_t*) i16_audio_in_data, (uint32_t) AUDIO_IN_BUFFER_LENGTH);
		ASA_general_settings.audio_input = ASA_AUDIO_INPUT_AUDIOJACK;
	  ASA_axis_settings.y_max = (int16_t)VOLTAGE_UPPER_LIMIT_DEFAULT;
	  ASA_axis_settings.y_min = (int16_t)VOLTAGE_LOWER_LIMIT_DEFAULT;
	  ASA_create_xy_axis();

	}
	else
	{
		state = 1;
		BSP_AUDIO_IN_Stop(CODEC_PDWN_HW);
		BSP_AUDIO_IN_DeInit();
		/*Initialize audio for recording from digital microphones: stereo, gain 80% (0dB), sampling frequency 44.1 kHz*/
		BSP_AUDIO_IN_Init(INPUT_DEVICE_DIGITAL_MICROPHONE_2, (uint8_t) 80, AUDIO_FREQUENCY_44K);
		/*Start recording*/
		BSP_AUDIO_IN_Record((uint16_t*) i16_audio_in_data, (uint32_t) AUDIO_IN_BUFFER_LENGTH);
		ASA_general_settings.audio_input = ASA_AUDIO_INPUT_MICROPHONE;
	  ASA_axis_settings.y_max = (int16_t)SPL_UPPER_LIMIT_DEFAULT;
	  ASA_axis_settings.y_min = (int16_t)SPL_LOWER_LIMIT_DEFAULT;
	  ASA_create_xy_axis();
	}

	return;
}

/*Read touch screen*/
static void ASA_read_ts_state(void)
{
	TS_StateTypeDef ts_state;
	BSP_TS_ResetTouchData(&ts_state);
	BSP_TS_GetState(&ts_state);

	if(ts_state.touchDetected)
		ASA_flags.ts_touch = 1;
	else
		ASA_flags.ts_touch = 0;

	return;
}

/*Initialize LCD*/
static void ASA_LCD_init(void)
{
	/* LCD clock configuration */
	/* PLLSAI_VCO Input = HSE_VALUE/PLL_M = 1 Mhz */
	/* PLLSAI_VCO Output = PLLSAI_VCO Input * PLLSAIN = 192 Mhz */
	/* PLLLCDCLK = PLLSAI_VCO Output/PLLSAIR = 192/5 = 38.4 Mhz */
	/* LTDC clock frequency = PLLLCDCLK / LTDC_PLLSAI_DIVR_4 = 38.4/4 = 9.6Mhz */
	RCC_PeriphCLKInitTypeDef PeriphClkInitStruct;
	PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_LTDC;
	PeriphClkInitStruct.PLLSAI.PLLSAIN = 192;
	PeriphClkInitStruct.PLLSAI.PLLSAIR = 5;
	PeriphClkInitStruct.PLLSAIDivR = RCC_PLLSAIDIVR_4;
	HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct);
	/* Configure LCD : Only one layer is used */
	BSP_LCD_Init();
	/* LCD Initialization */
	BSP_LCD_LayerDefaultInit(0, LCD_FB_START_ADDRESS);
	BSP_LCD_LayerDefaultInit(1,
	LCD_FB_START_ADDRESS + (BSP_LCD_GetXSize() * BSP_LCD_GetYSize() * 4));
	/* Enable the LCD */
	BSP_LCD_DisplayOn();
	/* Select the LCD Background Layer  */
	BSP_LCD_SelectLayer(0);
	/* Clear the Background Layer */
	BSP_LCD_Clear(LCD_COLOR_BLACK);
	/*Make the layer 0 visible*/
	BSP_LCD_SetLayerVisible(1, DISABLE);
	BSP_LCD_SetLayerVisible(0, ENABLE);
	/*Write text on LCD*/
	BSP_LCD_SetBackColor(LCD_COLOR_BLACK);
	BSP_LCD_SetTextColor(LCD_COLOR_YELLOW);

	ASA_create_xy_axis();

	return;
}

//TODO
/*Initialize audio facilities*/
static void ASA_audio_init(void)
{
	/*Initialize audio for recording from digital microphones: stereo, gain 80% (0dB), sampling frequency 44.1 kHz*/
	if(ASA_general_settings.audio_input == ASA_AUDIO_INPUT_MICROPHONE)
		BSP_AUDIO_IN_Init(INPUT_DEVICE_DIGITAL_MICROPHONE_2, (uint8_t) 80, AUDIO_FREQUENCY_44K);
	else
		BSP_AUDIO_IN_Init(INPUT_DEVICE_INPUT_LINE_1, (uint8_t) 80, AUDIO_FREQUENCY_44K);

	/*Start recording*/
	BSP_AUDIO_IN_Record((uint16_t*) i16_audio_in_data, (uint32_t) AUDIO_IN_BUFFER_LENGTH);

	return;
}

/*Initialize touch screen*/
static void ASA_ts_init(void)
{
	BSP_TS_Init(LCD_WIDTH, LCD_HEIGHT);

	return;
}

/*Initialize flags and settings*/
static void ASA_settings_init(void)
{
	ASA_flags.event = 0;
	ASA_flags.display_refresh = 0;
	ASA_flags.freq_axis_refresh = 0;
	ASA_flags.half_transfer = 0;
	ASA_flags.transfer_complete = 0;
	ASA_flags.ts_touch = 0;
	ASA_flags.ts_read_state = 0;

	ASA_general_settings.audio_input = ASA_AUDIO_INPUT_AUDIOJACK;
	ASA_axis_settings.x_max = (uint32_t)FREQUENCY_UPPER_LIMIT_DEFAULT;
	ASA_axis_settings.x_min = (uint32_t)FREQUENCY_LOWER_LIMIT_DEFAULT;
	ASA_axis_settings.y_max = (int16_t)VOLTAGE_UPPER_LIMIT_DEFAULT;
	ASA_axis_settings.y_min = (int16_t)VOLTAGE_LOWER_LIMIT_DEFAULT;

	return;
}

/*Initialize Audio Spectrum Analyzer*/
void ASA_init(void)
{
	ASA_settings_init();
	ASA_LCD_init();
	ASA_audio_init();
	ASA_ts_init();

	return;
}

/*Check flags and call corresponding tasks*/
void ASA_check_events(void)
{
	if(ASA_flags.event)
	{
		ASA_flags.event = 0;

		if(ASA_flags.display_refresh)
		{
			SCB_CleanInvalidateDCache();
			ASA_flags.display_refresh = 0;
			ASA_display_refresh();
			SCB_CleanInvalidateDCache();
		}

		if(ASA_flags.freq_axis_refresh)
		{
			SCB_CleanInvalidateDCache();
			ASA_flags.freq_axis_refresh = 0;
			ASA_freq_axis_refresh();
			SCB_CleanInvalidateDCache();
		}

		if(ASA_flags.half_transfer)
		{
			SCB_CleanInvalidateDCache();
			ASA_flags.half_transfer = 0;
			ASA_int2float_conv(f_audio_in_data, i16_audio_in_data, AUDIO_IN_BUFFER_LENGTH/2, ASA_INT2FLOAT_MODE_DEC2);
			ASA_make_third_octave_spectrum();
			SCB_CleanInvalidateDCache();
		}

		if(ASA_flags.transfer_complete)
		{
			SCB_CleanInvalidateDCache();
			ASA_flags.transfer_complete = 0;
			ASA_int2float_conv(f_audio_in_data, &i16_audio_in_data[AUDIO_IN_BUFFER_LENGTH/2], AUDIO_IN_BUFFER_LENGTH/2, ASA_INT2FLOAT_MODE_DEC2);
			ASA_make_third_octave_spectrum();
			SCB_CleanInvalidateDCache();
		}

		if(ASA_flags.ts_read_state)
		{
			ASA_flags.ts_read_state = 0;
			ASA_read_ts_state();
			if(ASA_flags.ts_touch)
			{
				SCB_CleanInvalidateDCache();
				ASA_flags.ts_touch = 0;
				ASA_change_input();
				SCB_CleanInvalidateDCache();
			}
		}
	}

	return;
}

void ASA_AUDIO_IN_TransferComplete_CallBack(void)
{
	SCB_CleanInvalidateDCache();
	ASA_flags.transfer_complete = 1;
	ASA_flags.event = 1;

	return;
}

void ASA_AUDIO_IN_HalfTransfer_CallBack(void)
{
	SCB_CleanInvalidateDCache();
	ASA_flags.half_transfer = 1;
	ASA_flags.event = 1;

	return;
}

void ASA_ISR_1ms(void)
{
	static uint32_t display_refresh_timer = 0;
	static uint32_t freq_axis_refresh_timer = 0;
	static uint32_t ts_read_state_timer = 0;

	if(++display_refresh_timer >= DISPLAY_REFRESH_PERIOD)
	{
		display_refresh_timer = 0;
		ASA_flags.display_refresh = 1;
		ASA_flags.event = 1;
	}

	if(++freq_axis_refresh_timer >= FREQ_AXIS_REFRESH_PERIOD)
	{
		freq_axis_refresh_timer = 0;
		ASA_flags.freq_axis_refresh = 1;
		ASA_flags.event = 1;
	}

	if(++ts_read_state_timer >= TS_SAMPLING_PERIOD)
	{
		ts_read_state_timer = 0;
		ASA_flags.ts_read_state = 1;
		ASA_flags.event = 1;
	}

	return;
}




